#ifndef CONSOLE_H_
#define CONSOLE_H_

//#include <iostream>
#include <distributions/DistTab.h>
#include <functions/FuncTab.h>
#include <model/Model.h>
#include <model/SymTab.h>
#include <model/ParseTree.h>

class Console {
  std::ostream &_out;
  std::ostream &_err;
  FuncTab _functab;
  DistTab _disttab;
  SymTab _symtab;
  Model *_model;
  std::vector<ParseTree*> *_pvariables;
  ParseTree *_prelations;
  unsigned int _iteration;
  void errorRecovery();
public:
  Console(std::ostream &out, std::ostream &err);
  ~Console();
  void checkModel(std::string const &filename);
  void compile(std::map<std::string, SArray> const &data_table,
	       unsigned int nchain = 1);
  void setParameters(std::map<std::string, SArray> const &init_table,
		     unsigned int chain);
  void initialize();
  void update (long n);
  void setMonitor(std::string const &name, Range const &range,
		  unsigned int thin); 
  void setMonitor(std::string const &name, unsigned int thin); 
  void clearMonitor(std::string const &name, Range const &range);
  void clearMonitor(std::string const &name);
  void dumpState(std::ostream &out, unsigned int chain, bool observed);
  void iter();
  void coda(std::ostream &index, std::ostream &output);
  void coda(std::string const &name, std::ostream &index, 
	    std::ostream &output);
  void coda(std::string const &name, Range const &range, 
	    std::ostream &index, std::ostream &output);
  Model const *model();
  void setSeed(unsigned int seed);
};

#endif /* CONSOLE_H_ */
