#include <config.h>
#include <distributions/DBern.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>

using std::vector;

DBern::DBern()
  : DistFinite("dbern", 1)
{}

DBern::~DBern()
{}

bool 
DBern::checkParameterDim (SArray const &value, 
			  vector<SArray const *> const &parameters) const
{
  return (parameters[0]->length() == 1) && (value.length() == 1);
}

bool DBern::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double prob = *parameters[0]->value();
  return  (prob > 0.0 && prob < 1.0);
}

double DBern::logLikelihood(long x, vector<SArray const *> const &parameters) const
{
  double prob = *parameters[0]->value();

  switch(x) {
  case 0:
    return prob == 1 ? -DBL_MAX : log(1 - prob);
  case 1:
    return prob == 0 ? -DBL_MAX : log(prob);
  default:
    return -DBL_MAX;
  }
}

long DBern::randomSample(vector<SArray const *> const &parameters) const
{
  double prob = *parameters[0]->value();
  return runif(0, 1) < prob ? 1 : 0;
}

long DBern::lowerLimit(vector<SArray const *> const &parameters) const
{
  return 0;
}

long DBern::upperLimit(vector<SArray const *> const &parameters) const
{
  return 1;
}
