#ifndef DBERN_H_
#define DBERN_H_

#include <distributions/DistFinite.h>

/**
 * @short Bernoulli distribution
 * <pre>
 * R ~ dbern(p)
 * f(r | p) = p^r * (1 - p)^(1 -r) ; r in 0:1
 * </pre>
 */
class DBern : public DistFinite {
 public:
  DBern();
  ~DBern();
  double logLikelihood(long x, std::vector<SArray const *> const &parameters) 
    const;
  long randomSample(std::vector<SArray const *> const &parameters) const;
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that R and p are scalar
   */
  bool checkParameterDim (SArray const &value, 
			  std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that p lies in the open interval (0,1)
   */
  bool checkParameterValue(std::vector<SArray const *> const &parameters) const;
};

#endif /* DBERN_H_ */
