#include <config.h>
#include <distributions/DBeta.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

#define MAX_RESAMPLE 10

DBeta::DBeta()
  : DistReal("dbeta", 2, DIST_PROPORTION)
{}

DBeta::~DBeta()
{}

static void getParameters(double &a, double &b, vector<SArray const *> const &parameters)
{
  a = *(parameters[0]->value()); 
  b = *(parameters[1]->value()); 
}

bool DBeta::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double a,b;
  getParameters(a, b, parameters);
  return (a > 0.0 && b > 0.0);
}

double DBeta::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double a, b;
  getParameters(a, b, parameters);
  return dbeta(x, a, b, give_log);
}

double DBeta::p(double q, vector<SArray const *> const &parameters, bool lower,
		   bool log_p) const
{
  double a, b;
  getParameters(a, b, parameters);
  return pbeta(q, a, b, lower, log_p);
}

double DBeta::q(double p, vector<SArray const *> const &parameters, bool lower,
		   bool log_p) const
{
  double a, b;
  getParameters(a, b, parameters);
  return qbeta(p, a, b, lower, log_p);
}

double DBeta::r(vector<SArray const *> const &parameters) const
{
  double a, b;
  getParameters(a, b, parameters);
  return rbeta(a,b);
}
