#ifndef DBIN_H_
#define DBIN_H_

#include <distributions/DistFinite.h>

/**
 * @short Binomial distribution
 * <pre>
 * R ~ dbin(p, n)
 * f(r|p,n) = n!/r!.(n - r)! p^r * (1 - p)^(n - r) ; r in 0:n
 * </pre>
 */
class DBin : public DistFinite {
 public:
  DBin();
  ~DBin();
  double logLikelihood(long x, std::vector<SArray const *> const &parameters) 
	  const;
  long randomSample(std::vector<SArray const *> const &parameters) const;
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that R, p and n are scalar
   */
  bool checkParameterDim (SArray const &value, 
			  std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that p lies in (0,1) and n > 1
   */
  bool checkParameterValue(std::vector<SArray const *> const &parameters) const;
};

#endif /* DBIN_H_ */
