#ifndef DCAT_H_
#define DCAT_H_

#include <distributions/DistFinite.h>

/**
 * @short Categorical distribution
 * <pre>
 * R ~ dcat(p[])
 * f(r|p[]) = p[r] ; r in 1:dim(p)
 * </pre>
 */
class DCat : public DistFinite {
public:
  DCat();
  ~DCat();
  double logLikelihood(long x, std::vector<SArray const *> const &parameters) 
    const;
  long randomSample(std::vector<SArray const *> const &parameters) const;
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that R is scalar and the length of p is at least 2
   */
  bool checkParameterDim(SArray const &value,
			 std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that all elements of p lie in the interval (0,1)
   * and sum to 1
   */
  bool checkParameterValue(std::vector<SArray const *> const &parameters) const;
};

#endif /* DCAT_H_ */
