#include <config.h>
#include <distributions/DChisq.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

DChisq::DChisq()
  : DistReal("dchisq", 1, DIST_POSITIVE_REAL)
{}

DChisq::~DChisq()
{}

static double getDF(vector<SArray const *> const &parameters)
{
  return *(parameters[0]->value()); 
}

bool DChisq::checkParameterValue(vector<SArray const *> const &parameters) const
{
  return (getDF(parameters) > 0);
}

double 
DChisq::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double df = getDF(parameters);
  return dchisq(x, df, give_log);
}

double 
DChisq::p(double q, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double df = getDF(parameters);
  return pchisq(q, df, lower, log_p);
}

double
DChisq::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
const
{
  double df = getDF(parameters);
  return qchisq(p, df, lower, log_p);
}

double DChisq::r(vector<SArray const *> const &parameters) const
{
  double df = getDF(parameters);
  return rchisq(df);
}
