#include <config.h>
#include <distributions/DDexp.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>

using std::vector;

DDexp::DDexp()
  : DistReal("ddexp", 2, DIST_REAL)
{}

DDexp::~DDexp()
{}

static void
getParameters(double &mu, double &scale, vector<SArray const *> const &parameters)
{
  mu = *(parameters[0]->value());
  /* dpqr functions for exponential distribution  are parameterized in
     terms of scale = 1/mean */
  scale = 1/(*(parameters[1]->value()));
}

bool DDexp::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double mu = 0, scale = 0;
  getParameters(mu, scale, parameters);
  return (scale > 0);
}

double DDexp::d(double x, vector<SArray const *> const &parameters, bool dolog)
  const
{
  double mu, scale;
  getParameters(mu, scale, parameters);

  double d = dexp(fabs(x - mu), scale, dolog);
  if (dolog)
    return d - log(2);
  else
    return d/2;
}

double 
DDexp::p(double x, vector<SArray const *> const &parameters, bool lower, bool dolog)
  const
{
  double p;
  double mu, scale;
  getParameters(mu, scale, parameters);

  if (!lower)
    x = 2 * mu - x;

  if (x < mu) {
    p = pexp(mu - x, scale, false, false)/2;
  }
  else {
    p = 0.5 + pexp(x - mu, scale, true, false)/2;
  }

  if (dolog)
    return log(p);
  else 
    return p;
}

double
DDexp::q(double p, vector<SArray const *> const &parameters, bool lower, bool dolog)
  const
{
  double mu, scale;
  getParameters(mu, scale, parameters);

  if (dolog)
    p = exp(p);
  if (!lower)
    p = 1 - p;

  if (p < 0.5) {
    return mu - qexp(2 * (0.5 - p), scale, false, false);
  }
  else {
    return mu + qexp(2 * (p - 0.5), scale, true, false);
  }
}

double DDexp::r(vector<SArray const *> const &parameters) const
{
  double mu, scale;
  getParameters(mu, scale, parameters);

  double u = runif(0,1);
  if (u < 0.5)
    return mu - rexp(scale);
  else
    return mu + rexp(scale);
}

