#include <config.h>
#include <distributions/DExp.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>
#include <algorithm>

using std::max;
using std::vector;

DExp::DExp()
  : DistReal("dexp", 1, DIST_POSITIVE_REAL)
{}

DExp::~DExp()
{}

static void getParameters(double &scale, vector<SArray const *> const &parameters)
{
  /* R functions are parameterized in terms of scale = 1/rate */
  scale = 1 / *(parameters[0]->value()); 
}

bool DExp::checkParameterValue(vector<SArray const *> const &parameters) const
{
  return ( *(parameters[0]->value()) > 0);
}

double
DExp::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double scale;
  getParameters(scale, parameters);
  return dexp(x, scale, give_log);
}

double 
DExp::p(double q, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double scale;
  getParameters(scale, parameters);
  return pexp(q, scale, lower, log_p);
}

double 
DExp::q(double p, vector<SArray const *> const &parameters, bool lower,  bool log_p)
  const
{
  double scale;
  getParameters(scale, parameters);
  return qexp(p, scale, lower, log_p);
}

double DExp::r(vector<SArray const *> const &parameters) const
{
  double scale;
  getParameters(scale, parameters);
  return rexp(scale);
}

