#include <config.h>
#include <distributions/DGamma.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

DGamma::DGamma()
  : DistReal("dgamma", 2, DIST_POSITIVE_REAL)
{}

DGamma::~DGamma()
{}

static 
void getParameters(double &shape, double &scale, vector<SArray const *> const &parameters)
{
  shape = *(parameters[0]->value());
  scale = 1 / *(parameters[1]->value());
}

bool DGamma::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double shape, scale;
  getParameters(shape, scale, parameters);
  return (shape > 0 && scale > 0);
}

double
DGamma::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double shape, scale;
  getParameters(shape, scale, parameters);

  return dgamma(x, shape, scale, give_log);
}

double
DGamma::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  double shape, scale;
  getParameters(shape, scale, parameters);

  return pgamma(q, shape, scale, lower, give_log);
}

double 
DGamma::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double shape, scale;
  getParameters(shape, scale, parameters);

  return qgamma(p, shape, scale, lower, log_p);
}

double DGamma::r(vector<SArray const *> const &parameters) const
{
  double shape, scale;
  getParameters(shape, scale, parameters);

  return rgamma(shape, scale);
}
