#include <config.h>
#include <distributions/DInterval.h>
#include <sarray/SArray.h>

#include <cfloat>
#include <algorithm>

using std::min;
using std::max;

using std::vector;

DInterval::DInterval()
  : DistFinite("dinterval", 2)
{
}

DInterval::~DInterval()
{}

bool 
DInterval::checkParameterDim(SArray const &value,
			     vector<SArray const *> const &parameters) const
{
  return (value.length() == 1) && (parameters[0]->length() == 1);
}

bool DInterval::checkParameterValue(vector<SArray const *> const &parameters) const
{
  unsigned long length = parameters[1]->length();
  double const *cutpoints = parameters[1]->value();
  for (unsigned long i = 1; i < length; ++i) {
    if (cutpoints[i] < cutpoints[i-1])
      return false;
  }
  return true;
}

long DInterval::lowerLimit(vector<SArray const *> const &parameters) const
{
  if (lowerBound(parameters) != 0) {
    long lower = static_cast<long>(*(lowerBound(parameters)->value()));
    return lower > 0 ? lower : 0;
  }
  else {
    return 0;
  }
}

long DInterval::upperLimit(vector<SArray const *> const &parameters) const
{
  if (upperBound(parameters) != 0) {
    long ub = static_cast<long>(*(upperBound(parameters)->value()) + 1.0E-6);
    return min(ub, parameters[1]->length());
  }
  else {
    return parameters[1]->length();
  }
}

double DInterval::logLikelihood(long x, vector<SArray const *> const &parameters) const
{
  if (x < lowerLimit(parameters) || x > upperLimit(parameters)) {
    return -DBL_MAX;
  }
  else {
    double t = *parameters[0]->value();
    double const *cutpoints = parameters[1]->value();

    if (x > 0 && t <= cutpoints[x-1])
      return -DBL_MAX;
    else if (x < parameters[1]->length() && t > cutpoints[x])
      return -DBL_MAX;
    else
      return 0;
  }
}

long DInterval::randomSample(vector<SArray const *> const &parameters) const
{
  /* The random sample from DInterval is not random at all,
     but deterministic.
  */
  double t = *parameters[0]->value();
  double const *cutpoints = parameters[1]->value();
  long length = parameters[1]->length();

  if (t < cutpoints[0])
    return 0;
  else {
    for (long i = 1; i < length; ++i) {
      if (t > cutpoints[i-1] && t <= cutpoints[i])
	return i;
    }
    return length;
  }
}
