#ifndef DINTERVAL_H_
#define DINTERVAL_H_

#include <distributions/DistFinite.h>

/**
 * @short Interval censored distribution
 * <pre>
 * i ~ dinterval(t, cutpoints[])
 * f(i|t) = 1 if t < cutpoints[i] and t >= cutpoints[i-1]
 *        = 0 otherwise
 * </pre>
 */
class DInterval : public DistFinite {
 public:
  DInterval();
  ~DInterval();
		
  double logLikelihood(long x, std::vector<SArray const *> const &parameters) const;
  long randomSample(std::vector<SArray const *> const &parameters) const;
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that i and t are scalar
   */
  bool checkParameterDim(SArray const &value,
			 std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that cutpoints are in ascending order
   */
  bool checkParameterValue(std::vector<SArray const *> const &parameters) const;
};

#endif /* DINTERVAL_H_ */
