#include <config.h>
#include <distributions/DLnorm.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>

using std::vector;

DLnorm::DLnorm()
  : DistReal("dlnorm", 2, DIST_POSITIVE_REAL)
{}

DLnorm::~DLnorm()
{}

static void
getParameters(double &meanlog, double &sdlog, vector<SArray const *> const &parameters)
{
  meanlog = *(parameters[0]->value());
  double tau = *(parameters[1]->value());
  sdlog = 1 / (tau * tau);
}

bool DLnorm::checkParameterValue(vector<SArray const *> const &parameters) const
{
  return *(parameters[1]->value()) > 0;
}

double 
DLnorm::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double meanlog, sdlog;
  getParameters(meanlog, sdlog, parameters);
  return dlnorm(x, meanlog, sdlog, give_log);
}

double 
DLnorm::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log) 
  const
{
  double meanlog, sdlog;
  getParameters(meanlog, sdlog, parameters);
  return plnorm(q, meanlog, sdlog, lower, give_log);
}

double 
DLnorm::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double meanlog, sdlog;
  getParameters(meanlog, sdlog, parameters);
  return qlnorm(p, meanlog, sdlog, lower, log_p);
}

double 
DLnorm::r(vector<SArray const *> const &parameters) const
{
  double meanlog, sdlog;
  getParameters(meanlog, sdlog, parameters);
  return rlnorm(meanlog, sdlog);
}
