#include <config.h>
#include <distributions/DLogis.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

DLogis::DLogis()
  : DistReal("dlogis", 2, DIST_REAL)
{}

DLogis::~DLogis()
{}

void 
getParameters(double &location, double &scale, vector<SArray const *> const &parameters)
{
  location = *(parameters[0]->value());
  // Use scale in place of precision
  scale = 1  / *(parameters[1]->value());
}

bool DLogis::checkParameterValue (vector<SArray const *> const &parameters) const
{
  return (*(parameters[1]->value()) > 0);
}

double 
DLogis::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double location, scale;
  getParameters(location, scale, parameters);

  return dlogis(x, location, scale, give_log);
}

double 
DLogis::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  double location, scale;
  getParameters(location, scale, parameters);

  return plogis(q, location, scale, lower, give_log);
}

double 
DLogis::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double location, scale;
  getParameters(location, scale, parameters);

  return qlogis(p, location, scale, lower, log_p);
}

double 
DLogis::r(vector<SArray const *> const &parameters) const
{
  double location, scale;
  getParameters(location, scale, parameters);

  return rlogis(location, scale);
}
