#ifndef DMNORM_H_
#define DMNORM_H_

#include <distributions/Distribution.h>

/**
 * @short Multivariate normal distribution
 * <pre>
 * x[] ~ dmnorm(mu[], T[,])
 * f(x | mu, T) = sqrt(det(T)) * exp(-1/2 * (x-mu) %*% T %*% t(x-mu))
 * </pre>
 */
class DMNorm : public Distribution {
public:
  DMNorm();
  ~DMNorm();

  double logLikelihood(SArray const &x,
		       std::vector<SArray const *> const &parameters) const;
  void randomSample(SArray &x,
		    std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that x, mu are vectors, T is a square matrix and the
   * sizes of mu, T and x conform.
   */
  bool checkParameterDim (SArray const &value,
			  std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that T is symmetric. Note that there is currently
   * NO CHECK that T is positive definite.
   */
  bool checkParameterValue (std::vector<SArray const *> const &parameters) const;
  /**
   * Convenience random sampler function 
   */
  static void randomsample(double *x, double const *mu, double const *tau,
			   int nrow);
};

#endif /* DMNORM_H_ */
