#ifndef DMSTATE_H_
#define DMSTATE_H_

#include <distributions/DistFinite.h>

/**
 * Distribution of x in multi-state homogeneous Markov model at time t
 * when state at time 0 was xold and transition intensity matrix is
 * Lambda
 * <pre> 
 * x ~ dmstate(xold, dt, Lambda[,]) 
 * </pre>
 * @short Interval censored transitions in a multistate Markov model
 */
class DMState : public DistFinite {
public:
  DMState();
  ~DMState();
  double logLikelihood(long x, std::vector<SArray const *> const &parameters) const;
  long randomSample(std::vector<SArray const *> const &parameters) const;
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that x is scalar, t is scalar and Lambda is
   * a square matrix
   */
  bool checkParameterDim(SArray const &value,
			 std::vector<SArray const *> const &parameters) const;
  /**
   * Checks that t > 0, xold is in permitted range 1:nrow(Lambda)
   * and Lambda is a valid transition intensity matrix with
   * rows that sum to 0.
   */
  bool checkParameterValue(std::vector<SArray const *> const &parameters) const;
};

#endif /* DMSTATE_H_ */
