#include <config.h>
#include <distributions/DNegBin.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>
#include <vector>

using std::vector;

DNegBin::DNegBin()
  : DistDiscrete("dnegbin", 2)
{}

DNegBin::~DNegBin()
{}

static void getParameters(double &p, double &n, vector<SArray const *> const & parameters)
{
  p = *(parameters[0]->value());
  n = *(parameters[1]->value());
}

bool 
DNegBin::checkParameterDim (SArray const &value, 
			  vector<SArray const *> const &parameters) const
{
  return (parameters[0]->length() == 1) && (parameters[1]->length() == 1)
    && (value.length() == 1);
}

bool DNegBin::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double p=0, n=0;
  getParameters(p, n, parameters);
  return (n > 0 && p > 0 && p < 1);
}

double
DNegBin::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double p = 0, n = 0;
  getParameters(p, n, parameters);
  return dnbinom(x, n, p, give_log);
}

double
DNegBin::p(double q, vector<SArray const *> const &parameters, bool lower, 
	      bool give_log) const
{
  double p = 0, n = 0;
  getParameters(p, n, parameters);
  return pnbinom(q, n, p, lower, give_log);
}

double 
DNegBin::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double prob = 0, n = 0;
  getParameters(prob, n, parameters);
  return qnbinom(prob, n, p, lower, log_p);
}

double 
DNegBin::r(vector<SArray const *> const &parameters)
  const
{
  double p = 0, n = 0;
  getParameters(p, n, parameters);
  return rnbinom(n, p);
}
