#include <config.h>
#include <distributions/DNorm.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>

using std::vector;

DNorm::DNorm()
  : DistReal("dnorm", 2, DIST_REAL)
{}

DNorm::~DNorm()
{}

static void 
getParameters(double &mu, double &sigma, vector<SArray const *> const &parameters)
{
  mu = *(parameters[0]->value());
  sigma  = 1 / sqrt(*parameters[1]->value());
}

bool DNorm::checkParameterValue(vector<SArray const *> const &parameters) const
{
  return (*(parameters[1]->value()) > 0);
}

double
DNorm::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double mu = parameters[0]->value()[0];
  double sigma = 1 / sqrt(parameters[1]->value()[0]);
  return dnorm(x, mu, sigma, give_log);
}

double
DNorm::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  double mu, sigma;
  getParameters(mu, sigma, parameters);
  return pnorm(q, mu, sigma, lower, give_log);
}

double 
DNorm::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double mu, sigma;
  getParameters(mu, sigma, parameters);
  return qnorm(p, mu, sigma, lower, log_p);
}

double 
DNorm::r(vector<SArray const *> const &parameters)
  const
{
  double mu, sigma;
  getParameters(mu, sigma, parameters);
  return rnorm(mu, sigma);
}

