#include <config.h>
#include <distributions/DPar.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>
#include <cfloat>

using std::vector;

DPar::DPar()
  : DistReal("dpar", 2, DIST_POSITIVE_REAL, 2, 0)
{}

DPar::~DPar()
{}

static 
void getParameters(double &alpha, double &c, vector<SArray const *> const &parameters)
{
  alpha = *(parameters[0]->value());
  c = *(parameters[1]->value());
}

bool DPar::checkParameterValue(vector<SArray const *> const &parameters) const
{
  double alpha=0, c= 0;
  getParameters(alpha, c, parameters);
  return (alpha > 0 && c > 0);
}

double 
DPar::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double alpha, c;
  getParameters(alpha, c, parameters);

  if (x < c)
    return give_log ? -DBL_MAX : 0;

  if (give_log)
    return log(alpha) + alpha * log(c) - (alpha + 1) * log(x);
  else
    return alpha * exp(alpha * log(c) - (alpha + 1) * log(x));
}

double 
DPar::p(double x, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  double alpha, c;
  getParameters(alpha, c, parameters);
  
  if (x < c)
    return give_log ? -DBL_MAX : 0;
  
  double logq = alpha * log(c/x);
  if (!lower) {
    return give_log ? logq : exp(logq);
  }
  else {
    return give_log ? log(1 - exp(logq)) : 1 - exp(logq);
  }
}

double 
DPar::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double alpha, c;
  getParameters(alpha, c, parameters);

  if ((log_p  && p > 0) ||                    
      (!log_p && (p < 0 || p > 1)) )          
    return c; /* FIXME: should return NaN */

  double logp;

  if (lower) {
    if (log_p)
      logp = p;
    else
      logp = log(p);
  }
  else {
    if (log_p)
      logp = log(1 - exp(p)); 
    else
      logp = log(1 - p);
  }
  
  return exp(log(c) - logp/alpha);
}

double DPar::r(vector<SArray const *> const &parameters) const
{
  double alpha, c;
  getParameters(alpha, c, parameters);

  double logp = rexp(1);
  return exp(log(c) - logp/alpha);
}
