#include <config.h>
#include <distributions/DPois.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

DPois::DPois()
  : DistDiscrete("dpois", 1)
{}

DPois::~DPois()
{}

bool DPois::checkParameterDim (SArray const &value, 
			       vector<SArray const *> const &parameters) const
{
  return (value.length() == 1) && (parameters[0]->length() == 1);
}

bool DPois::checkParameterValue(vector<SArray const *> const &parameters) const
{
  return (*parameters[0]->value() > 0);
}

double
DPois::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  return dpois(x, *parameters[0]->value(), give_log);
}

double
DPois::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  return ppois(q, *parameters[0]->value(), lower, give_log);
}

double 
DPois::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  return qpois(p, *parameters[0]->value(), lower, log_p);
}

double 
DPois::r(vector<SArray const *> const &parameters)
  const
{
  return rpois(*parameters[0]->value());
}
