#include <config.h>
#include <distributions/DT.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>

using std::vector;

DT::DT()
  : DistReal("dt", 3, DIST_REAL)
{}

DT::~DT()
{}

static void getParameters(double &mu, double &tau, double &df, 
			  vector<SArray const *> const &parameters)
{
  mu = *(parameters[0]->value()); // Mean
  tau = *(parameters[1]->value()); // Precision
  df = *(parameters[2]->value()); // Degrees of freedom
}

bool DT::checkParameterValue (vector<SArray const *> const &parameters) const
{
  return (*(parameters[1]->value()) > 0 && *(parameters[2]->value()) > 0);
}

double DT::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double mu, tau, df;
  getParameters(mu, tau, df, parameters);
  x = (x - mu) * sqrt(tau);
  if (give_log) {
    //return -((df+1)/2) * log(1 + (x*x)/df) + log(tau)/2;
    return dt(x, df, 1) + log(tau)/2;
  }
  else {
    return dt(x, df, 0) * sqrt(tau);
  }
}

double DT::p(double x, vector<SArray const *> const &parameters, bool lower, 
		bool use_log) const
{
  double mu, tau, df;
  getParameters(mu, tau, df, parameters);
  x = (x - mu) * sqrt(tau);
  return pt(x, df, lower, use_log);
}

double DT::q(double p, vector<SArray const *> const &parameters, bool lower, 
		bool log_p) const
{
  double mu, tau, df;
  getParameters(mu, tau, df, parameters);
  return mu + qt(p, df, lower, log_p) / sqrt(tau);
}

double DT::r(vector<SArray const *> const &parameters) const
{
  double mu, tau, df;
  getParameters(mu, tau, df, parameters);
  return rt(df) / sqrt(tau) + mu;
}
