#include <config.h>
#include <distributions/DUnif.h>
#include <sarray/SArray.h>

#include <Rmath.h>

using std::vector;

DUnif::DUnif()
  : DistReal("dunif", 2, DIST_REAL, 1, 2)
{}


DUnif::~DUnif()
{}

static void getParameters(double &a, double &b, vector<SArray const *> const &parameters)
{
  a = *(parameters[0]->value()); 
  b = *(parameters[1]->value()); 
}

bool DUnif::checkParameterValue (vector<SArray const *> const &parameters) const
{
  double a = 0, b = 0;
  getParameters(a,b,parameters);
  return (a < b);
}

double
DUnif::d(double x, vector<SArray const *> const &parameters, bool give_log) const
{
  double a, b;
  getParameters(a, b, parameters);
  return dunif(x, a, b, give_log);
}

double
DUnif::p(double q, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double a, b;
  getParameters(a, b, parameters);
  return punif(q, a, b, lower, log_p);
}

double
DUnif::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double a, b;
  getParameters(a, b, parameters);
  return qunif(p, a, b, lower, log_p);
}

double DUnif::r(vector<SArray const *> const &parameters) const
{
  double a, b;
  getParameters(a, b, parameters);
  return runif(a, b);
}
