#include <config.h>
#include <distributions/DWeib.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <vector>
#include <cmath>

using std::vector;

DWeib::DWeib()
  : DistReal("dweib", 2, DIST_POSITIVE_REAL)
{}

DWeib::~DWeib()
{}

static void
getParameters(double &shape, double &scale, vector<SArray const *> const &parameters)
{
  shape = *(parameters[0]->value());
  double lambda = *(parameters[1]->value());
  double logscale = -log(lambda)/shape;
  //scale = pow(lambda, -1/shape);
  scale = exp(logscale);
}

bool DWeib::checkParameterValue (vector<SArray const *> const &parameters) const
{
  return (*(parameters[0]->value()) > 0 && *(parameters[1]->value()) > 0);
}

double 
DWeib::d(double x, vector<SArray const *> const &parameters, bool give_log)
  const
{
  double shape, scale;
  getParameters(shape, scale, parameters);
  return dweibull(x, shape, scale, give_log);
}

double
DWeib::p(double q, vector<SArray const *> const &parameters, bool lower, bool give_log)
  const
{
  double shape, scale;
  getParameters(shape, scale, parameters);
  return pweibull(q, shape, scale, lower, give_log);
}

double 
DWeib::q(double p, vector<SArray const *> const &parameters, bool lower, bool log_p)
  const
{
  double shape, scale;
  getParameters(shape, scale, parameters);
  return qweibull(p, shape, scale, lower, log_p);
}

double DWeib::r(vector<SArray const *> const &parameters) const
{
  double shape, scale;
  getParameters(shape, scale, parameters);
  return rweibull(shape, scale);
}

