#include <config.h>
#include <distributions/DistDiscrete.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <stdexcept>
#include <cfloat>
#include <cmath>
#include <algorithm>

using std::max;
using std::min;
using std::string;
using std::vector;
using std::length_error;

DistDiscrete::DistDiscrete(string const &name, unsigned int npar)
  : Distribution(name, npar)
{
}

double DistDiscrete::logLikelihood(SArray const &x,
				   vector<SArray const *> const &parameters) const
{
  /* FIXME: This code is identical to DistReal::logLikelihood */

  double y = x.value()[0];

  bool bb = lowerBound(parameters) != 0; 
  bool ba = upperBound(parameters) != 0;
  
  double lower, upper;
  if (bb || ba) {
    lower = lowerLimit(parameters);
    upper = upperLimit(parameters);
    if (y < lower || y > upper) {
      return -DBL_MAX;
    }
  }
    
  double loglik = d(y, parameters, true);
  if (bb && ba) {
    loglik -= log(p(upper, parameters, true, false) - 
		  p(lower, parameters, true, false));
  }
  else if (bb) {
    loglik -= p(lower - 1, parameters, false, true);
  }
  else if (ba) {
    loglik -= p(upper, parameters, true, true);
  }
  return loglik;
}

void DistDiscrete::randomSample(SArray &x,
				vector<SArray const *> const &parameters) const
{
  if (x.length() != 1)
    throw length_error("length mismatch in DistDiscrete::randomSample");

  bool bb = lowerBound(parameters) != 0;
  bool ba = upperBound(parameters) != 0;
  
  double y;
  if (ba || bb) {
    double plower = p(lowerLimit(parameters) - 1, parameters, true, false);
    double pupper = p(upperLimit(parameters), parameters, true, false);
    double u = runif(plower, pupper);
    y = q(u, parameters, true, false);
  }
  else {
    y = r(parameters);
  }
  x.setValue(&y,1);
  
}

long DistDiscrete::lowerLimit(vector<SArray const *> const &parameters) const
{
  SArray const *pl = lowerBound(parameters);
  if (pl) {
    long ll = (long) (*(pl->value()) + 0.5);
    return ll > 0 ? ll : 0;
  }
  else
    return 0;
}

long DistDiscrete::upperLimit(vector<SArray const *> const &parameters) const
{
  SArray const *pu = upperBound(parameters);
  if (pu) {
    return (long) *(pu->value());
  }
  else
    return LONG_MAX;
}
