#include <config.h>
#include <distributions/DistFinite.h>
#include <sarray/SArray.h>

#include <cfloat>
#include <stdexcept>

using std::string;
using std::vector;
using std::length_error;

DistFinite::DistFinite(string const &name, unsigned int npar)
  : Distribution(name, npar) 
{
}

double 
DistFinite::logLikelihood(SArray const &x, 
			  vector<SArray const *> const &parameters) const
{
  long y = static_cast<long>(x.value()[0] + DBL_EPSILON);
  return logLikelihood(y, parameters);
}
	
void
DistFinite::randomSample(SArray &x, vector<SArray const *> const &parameters)
  const
{
  if (x.length() != 1)
    throw length_error ("SArray length not 1 in call to DistFinite::randomSample");
  
  double y = static_cast<double>(randomSample(parameters));
  x.setValue(&y, 1);
}

DistFinite const *asFinite(Distribution const *dist)
{
	return dynamic_cast<DistFinite const*>(dist);
}
