#include <config.h>
#include <functions/ILogit.h>
#include <sarray/SArray.h>

#include <Rmath.h>

#include <cmath>
#include <cfloat>

using std::vector;

ILogit::ILogit ():InverseLinkFunc ("ilogit", "logit")
{
}

double
ILogit::evaluate (vector < SArray const *>const &args) const
{
  /* Returning 0 or 1 can cause numerical problems. A result
     of 0 or 1 may just be the result of numerical imprecision.
     What we really want to return is e  or (1 - e) for very
     small e */

  double q = *(args[0]->value ());
  if (!R_FINITE (q)) {
    return q > 0 ? 1 : 0;
  }

  double y = exp (-q);
  if (!R_FINITE (y))
    {
      return DBL_EPSILON;
    }
  else
    {
      double p =  1 / (1 + y);
      return p == 1 ? 1 - DBL_EPSILON : p;
    }
}
