#ifndef CONSTANT_NODE_H_
#define CONSTANT_NODE_H_

#include <graph/Node.h>

/**
 * Constant nodes represent the top level parameters in any directed
 * graphical model.  They are always scalar, and should not have
 * parents.  It is not an error to add parents to a constant node, but
 * it does not make any sense either.
 *
 * The value of a constant node is fixed after initialization.
 */
class ConstantNode : public Node {
 public:
  /**
   * Default constructor. It is assumed that the value of the node
   * will be set later. 
   */
  ConstantNode();
  /**
   * Construct node and set its value. The value is fixed.
   */
  ConstantNode(double value);
  /**
   * This function does nothing. It exists only so that objects of
   * class ConstantNode can be instantiated.
   */
  void forwardSample();
};

bool isConstant(Node const *node);
ConstantNode *asConstant(Node *node);

#endif /* CONSTANT_NODE_H_ */




