#include <config.h>
#include <graph/LogicalNode.h>
#include <functions/Function.h>
#include <graph/NodeError.h>

#include <vector>
#include <string>
#include <math.h>

using std::vector;
using std::string;

//debuggin
#include <iostream>
using std::cout;
using std::endl;

LogicalNode::LogicalNode(Index const &dim)
  : DeterministicNode(dim), _func(0), _parameters()
{
}

LogicalNode::~LogicalNode()
{
}

static string makeName(Function const *func,
		       vector<Node*> const &parameters)
{
  string name;

  string const &fname = func->name();
  if (fname == "*" || fname == "/" || fname == "+" || fname == "-")
    {
      for (unsigned int i = 0; i < parameters.size(); ++i) {
	if (i > 0) {
	  name.append(fname);
	}
	name.append(parameters[i]->name());
      }
    }
  else
    {
      name.append(fname);
      name.append("(");
      for (unsigned int i = 0; i < parameters.size(); ++i) {
	if (i > 0)
	  name.append(",");
	name.append(parameters[i]->name());
      }
      name.append(")");
    }

  return name;
}

void LogicalNode::setParameters(vector<Node *> const &parameters)
{
  if (_func == 0) {
    throw NodeError(this,"Attempt to set parameters of LogicalNode with no function");
  }
  if (!_parameters.empty()) {
    throw NodeError(this, "Attempt to reset parameters of LogicalNode");
  }
  
  for (unsigned int i = 0; i < parameters.size(); i++) {
    _parameters.push_back(&parameters[i]->data);
    this->addParent(parameters[i]);
  }

  if (!_func->checkParameterLength(_parameters)) {
    throw NodeError(this, "Incorrect number of parameters in LogicalNode");
  }
  if (!_func->checkDim(data, _parameters)) {
    throw NodeError(this, "Non-conforming parameters in logical node");
  }

  if (this->name().empty()) {
    string name = makeName(_func, parameters);
    setName(name);
  }
}

vector<SArray const *> const &LogicalNode::parameters() const
{
  return _parameters;
}

void LogicalNode::setFunction (Function const *func)
{
  _func = func;
}

Function const* LogicalNode::function() const
{
  return _func;
}

void LogicalNode::forwardSample()
{
  if (!_func->checkParameterValue(_parameters)) {
    throw NodeError(this, "Invalid parameter values for LogicalNode");
  }
  _func->evaluate(data, _parameters);
}

void LogicalNode::initialize()
{
  if (_func == 0) {
    throw NodeError(this, 
		    "Attempt to initialize LogicalNode with no function");
  }
  else if (_parameters.empty()) {
    throw NodeError(this,
		    "Attempt to initialize LogicalNode with no parameters");
  }
  else {
    DeterministicNode::initialize();
  }
}

LogicalNode *asLogical(Node *node)
{
  return dynamic_cast<LogicalNode*>(node);
}

bool isLogical(Node const *node)
{
  return dynamic_cast<LogicalNode const*>(node);
}
