#include <config.h>
#include <graph/MixtureNode.h>

#include <utility>
#include <vector>
#include <cfloat>
#include <stdexcept>

using std::pair;
using std::vector;
using std::map;
using std::invalid_argument;
using std::logic_error;

#include <iostream>
using std::cout;
using std::endl;

MixtureNode::MixtureNode(Node *index,
		 vector<pair<long, Node *> > const &parameters)
  : DeterministicNode(parameters[0].second->data.dim(true)), _index(index)
{
  if (index->data.length() != 1) {
    throw invalid_argument("Invalid index parameter for mixture node");
  }
  this->addParent(index);

  unsigned int ndim = parameters.size();
  Index const &default_dim = data.dim(false);
  for (unsigned int i = 0; i < ndim; ++i) {
    if (!parameters[i].second) {
      throw invalid_argument("Null parameter in MixtureNode");
    }
    if (parameters[i].second->data.dim(true) != default_dim) {
      throw invalid_argument("Dimension mismatch for MixtureNode parameters");
    }
    /*
    cout << "Adding node " << parameters[i].second->name() << " with key "
	 << parameters[i].first << endl;
    */
    this->addParent(parameters[i].second);
    _map[parameters[i].first] = parameters[i].second;
  }
  //cout << endl;
}

MixtureNode::~MixtureNode()
{
}

void MixtureNode::forwardSample()
{
  long i = static_cast<long>(*_index->data.value() + DBL_EPSILON);
  map<long,Node*>::iterator p = _map.find(i);
  if (p != _map.end()) {
    data.setValue(p->second->data.value(), 
		  p->second->data.length());
  }
  else {
    throw logic_error("Invalid index in MixtureNode");
  }
}

Node const *MixtureNode::index() const
{
  return _index;
}

MixtureNode *asMixture(Node *node)
{
  return dynamic_cast<MixtureNode*>(node);
}

bool isMixture(Node const *node)
{
  return dynamic_cast<MixtureNode const*>(node);
}
