#ifndef MIXTURE_NODE_H_
#define MIXTURE_NODE_H_

#include <graph/DeterministicNode.h>
#include <map>
#include <vector>

/**
 * A mixture node takes its value from one of several parents, based
 * on the current value of an index node, which is also a parent
 * @short Node for mixture models
 */
class MixtureNode : public DeterministicNode {
	std::map<long, Node*> _map;
	Node *_index;
 public:
	/**
	 * Constructor. 
	 * @param index The index node is a scalar node that takes integer
	 * values.
	 * @param parameters Vector of pairs. Each pair associates
	 * a possible value of the index node with a parent. When 
	 * the value of the index node is the first member of the pair,
	 * then the value of the node is copied from the corresponding 
	 * parent.
	 */
	MixtureNode(Node *index, 
		    std::vector<std::pair<long,Node *> > const &parameters);
	~MixtureNode();
	
	void forwardSample();
	/**
	 * Returns a pointer to the index node.
	 */
	Node const *index() const;
};

MixtureNode *asMixture(Node *node);
bool isMixture(Node const *node);

#endif /* MIXTURE_NODE_H_ */
