#include <config.h>
#include <model/LogicalFactory.h>
#include <graph/Node.h>
#include <graph/LogicalNode.h>
#include <functions/Function.h>

#include <stdexcept>
#include <string>

using std::pair;
using std::map;
using std::vector;
using std::length_error;
using std::invalid_argument;
using std::string;

LogicalNode*
LogicalFactory::getLogicalNode(Function const *func, 
			       vector<Node*> const &parents)
{
  if (func == 0) {
    throw invalid_argument("NULL function passed to getLogicalNode");
  }
  map<pair<Function const *, vector<Node*> >, LogicalNode*, 
    ltlogical>::iterator i = 
    _logicalmap.find(pair<Function const*, vector<Node*> >(func,parents));

  if (i == _logicalmap.end()) {
    // Create a new logical node

    /* 
       In order to calculate the size of the new node, we
       need to turn the vector of Node pointers into a
       vector of SArray pointers, so we can call Function::dim.
    */
    vector<SArray const *> parameters(parents.size()); 
    long j = 0;
    for (vector<Node*>::const_iterator p = parents.begin();
	 p != parents.end(); ++p)
      {
	parameters[j++] = &((*p)->data);
      }
    if (!func->checkParameterLength(parameters)) {
      string msg("Incorrect number of parameters for function");
      msg.append(func->name());
      throw length_error(msg);
    }

    LogicalNode *lnode = new LogicalNode(func->dim(parameters));
    _graph.add(lnode);

    lnode->setFunction(func);
    lnode->setParameters(parents);
    _logicalmap[pair<Function const*, vector<Node*> >(func, parents)] = 
      lnode;
    return lnode;
  }
  else {
    // Return a previously created node
    return i->second;
  }
}

Graph const &LogicalFactory::graph() const
{
  return _graph;
}
