#include <config.h>
#include <model/SymTab.h>

#include <cfloat>
#include <string>
#include <stdexcept>
#include <utility>

using std::vector;
using std::map;
using std::pair;
using std::string;
using std::runtime_error;

#include <iostream>
using std::cout;
using std::endl;

SymTab::SymTab()
{
}

void SymTab::addVariable(string const &name, Index const &dim)
{
  if (_varTable.find(name) != _varTable.end()) {
    string msg("Name ");
    msg.append(name);
    msg.append(" already in use in symbol table");
    throw runtime_error(msg);
  }

  NodeArray *array = new NodeArray(dim);
  array->setName(name);
  _varTable[name] = array;
}

NodeArray* SymTab::getVariable(string const &name) const
{
  map<string, NodeArray*>::const_iterator p =  _varTable.find(name);
  
  if (p == _varTable.end()) {
    return 0;
  }
  else {
    return p->second;
  }
}

void SymTab::getNodes(std::vector<Node*> &nodes)
{
  map<string, NodeArray*>::const_iterator p(_varTable.begin());

  for (; p != _varTable.end(); ++p) {
    Graph const &graph = p->second->graph();
    graph.getNodes(nodes);
  }
}


void SymTab::writeData(std::map<std::string, SArray> const &data_table,
		       bool observed)
{
  for(map<string, SArray>::const_iterator p(data_table.begin()); 
      p != data_table.end(); ++p) {
    NodeArray *array = getVariable(p->first);
    if (array) {
      if (array->range().dim(false) != p->second.dim(false)) {
	string msg("Dimension mismatch when setting sarray of ");
	msg.append(p->first);
	throw runtime_error(msg);
      }
      array->setValue(p->second, observed);
    }
  }
}

void SymTab::readData(std::map<std::string, SArray> &data_table,
		      bool observed) const
{
  for (std::map<string, NodeArray*>::const_iterator p(_varTable.begin());
       p != _varTable.end(); ++p) {
    string const &name = p->first;
    if (data_table.find(name) != data_table.end()) {
      data_table.erase(name);
    }
    NodeArray const *array = p->second;
    SArray sarray(array->range().dim(false));
    array->getValue(sarray, observed);
    /* If array consists completely of missing sarrays, we don't
       bother to write it out */
    bool missing = true;
    for (long i = 0; i < sarray.length(); i++) {
      if (sarray.value()[i] != JAGS_NA) {
	missing = false;
	break;
      }
    }
    if (!missing) {
      data_table.insert(pair<string,SArray>(name, sarray));
    }
  }
}
