%{
/* Note the use of trailing context in the rules for the functions,
   allowing function names to be used as variable names.  This occurs
   in the "otree" example where "mean" is used as a variable name.
*/

#include <model/ParseTree.h>
#include <model/parser.h>
%}

EXPONENT	[eE][+-][0-9]+
BRACKET		[ \t]*\(

%option nounput
%option yylineno

%%
"model"			return MODEL;
"var"			return VAR;
"data"			return DATA;
"inits"			return INITS;

"for"/{BRACKET}        	return FOR;
"in"/{BRACKET}		return IN;
"in"                    return IN;
";"			return ';';
","			return ',';
":"			return ':';
"["			return '[';
"]"			return ']';
"("			return '(';
")"			return ')';
"{"			return '{';
"}"			return '}';
"="			return '=';
"~"			return '~';
"<-"			return ARROW;
"+"			return '+';
"-"			return '-';
"*"			return '*';
"/"			return '/';

"T"/{BRACKET}           return 'T';


[ \t\r\n]+              /* Eat whitespace */
"#".*\n                 /* Eat comments */

([0-9]+){EXPONENT} {
  yylval.val = atof(yytext); return DOUBLE;
}
([0-9]+"."[0-9]*){EXPONENT} {
  yylval.val = atof(yytext); return DOUBLE;
}
([0-9]+"."[0-9]*) {
  yylval.val = atof(yytext); return DOUBLE;
}
("."[0-9]+){EXPONENT} {
  yylval.val = atof(yytext); return DOUBLE;
}
("."[0-9]+) {
  yylval.val = atof(yytext); return DOUBLE;
}
[0-9]+ {
  yylval.val = atof(yytext); return DOUBLE;
}

([a-zA-Z]+)/{BRACKET} {
  yylval.stringptr = new std::string(yytext);
  return FUNC;
}

[a-zA-Z]+[a-zA-Z0-9\.]*	 {
  yylval.stringptr = new std::string(yytext);
  return NAME;
}

<<EOF>>	yyterminate();

%%

int yywrap()
{
  return 1;
}
