#ifndef CONJUGATE_DIRICHLET_H_
#define CONJUGATE_DIRICHLET_H_

#include <sampler/ConjugateSampler.h>

/**
 * Conjugate sampler for Dirichlet distribution.  A conjugate sampler
 * may be created for the stochastic node "snode" if it has a
 * Dirichlet distribution and its informative marginal stochastic
 * children have either a categorical distribution (dcat) or a
 * multinomial distribution (dmult) with snode as the probability
 * vector.  In the case of a multinomial distribution, the sample size
 * must not depend on snode.  The informative immediate deterministic
 * children must be either identity functions or mixture nodes.
 */
class ConjugateDirichlet : public ConjugateSampler {
 public:
  ConjugateDirichlet(StochasticNode *snode);
  ~ConjugateDirichlet();
  
  static bool canSample(StochasticNode const *snode);
  void update();
};

#endif /* CONJUGATE_DIRICHLET_H_ */
