#include <config.h>
#include <sampler/ConjugateFactory.h>
#include <sampler/ConjugateNormal.h>
#include <sampler/ConjugateGamma.h>
#include <sampler/ConjugateBeta.h>
#include <sampler/ConjugateDirichlet.h>
#include <sampler/ConjugateMNormal.h>
#include <sampler/ConjugateWishart.h>
#include <graph/StochasticNode.h>
#include <distributions/Distribution.h>

#include <stdexcept>
#include <string>
#include <map>

using std::string;
using std::map;
using std::invalid_argument;

ConjugateFactory::ConjugateFactory()
{
  _func_table["dnorm"] = ConjugateNormal::canSample;
  _func_table["dgamma"] = ConjugateGamma::canSample;
  _func_table["dexp"] = ConjugateGamma::canSample;
  _func_table["dchisq"] = ConjugateGamma::canSample;
  _func_table["dbeta"] = ConjugateBeta::canSample;
  _func_table["ddirch"] = ConjugateDirichlet::canSample;
  _func_table["dmnorm"] = ConjugateMNormal::canSample;
  _func_table["dwish"] = ConjugateWishart::canSample;
};

bool ConjugateFactory::canSample(StochasticNode const * snode) const
{
  map<const string, bool (*)(StochasticNode const*)>::const_iterator
    p = _func_table.find(snode->distribution()->name());

  if (p == _func_table.end())
    return false;
  else {
    return p->second(snode);
  }
}

GibbsSampler *ConjugateFactory::makeGibbsSampler(StochasticNode *snode) const
{
  switch (ConjugateSampler::getDist(snode)) {
  case NORM:
    return new ConjugateNormal(snode);
    break;
  case GAMMA: case EXP: case CHISQ:
    return new ConjugateGamma(snode);
    break;
  case BETA:
    return new ConjugateBeta(snode);
    break;
  case DIRCH:
    return new ConjugateDirichlet(snode);
    break;
  case MNORM:
    return new ConjugateMNormal(snode);
    break;
  case WISH:
    return new ConjugateWishart(snode);
    break;
  default:
    throw invalid_argument("Unable to create conjugate sampler");
  }
}
