#include <config.h>
#include <sampler/ConjugateSampler.h>
#include <graph/LogicalNode.h>
#include <graph/StochasticNode.h>
#include <functions/Function.h>
#include <distributions/Distribution.h>

#include <map>

using std::map;
using std::string;

ConjugateSampler::ConjugateSampler(StochasticNode *snode)
  : GibbsSampler(snode)
{
}

static map<const string, ConjugateDist> _dist_table;
static map<const string, ConjugateOp> _op_table;
static bool initialized = false;

/* FIXME: Tables should be static members of ConjugateSampler that
   are initialized at the start of the program */

static void init_tables ()
{
  _dist_table["dbern"] = BERN;
  _dist_table["dbeta"] = BETA;
  _dist_table["dbin"] = BIN;
  _dist_table["dcat"] = CAT;
  _dist_table["dchisq"] = CHISQ;
  _dist_table["ddexp"] = DEXP;
  _dist_table["ddirch"] = DIRCH;
  _dist_table["dexp"] = EXP;
  _dist_table["dgamma"] = GAMMA;
  _dist_table["dlnorm"] = LNORM;
  _dist_table["dlogis"] = LOGIS;
  _dist_table["dmnorm"] = MNORM;
  _dist_table["dmulti"] = MULTI;
  _dist_table["dnegbin"] = NEGBIN;
  _dist_table["dnorm"] = NORM;
  _dist_table["dpar"] = PAR;
  _dist_table["dpois"] = POIS;
  _dist_table["dt"] = T;
  _dist_table["dunif"] = UNIF;
  _dist_table["dweib"] = WEIB;
  _dist_table["dwish"] = WISH;

  _op_table["*"] = MULTIPLY;
  _op_table["/"] = DIVIDE;
  _op_table["+"] = ADD;
  _op_table["-"] = SUBTRACT;
  _op_table["UnaryMinus"] = UMINUS;
  _op_table["identity"] = IDENTITY;
  _op_table["inprod"] = INPROD;
  
  initialized = true;
};

ConjugateDist ConjugateSampler::getDist(StochasticNode const *snode)
{
  if(!initialized)
    init_tables();
  
  map<const string, ConjugateDist>::iterator p = 
    _dist_table.find(snode->distribution()->name());

  if (p == _dist_table.end())
    return OTHERDIST;
  else
    return p->second;
}

ConjugateOp ConjugateSampler::getOp(LogicalNode const *lnode)
{
  if(!initialized)
    init_tables();
  
  map<const string, ConjugateOp>::iterator p = 
    _op_table.find(lnode->function()->name());

  if (p == _op_table.end())
    return OTHEROP;
  else
    return p->second;
}
