#ifndef GIBBS_FACTORY_H_
#define GIBBS_FACTORY_H_

#include <sampler/SamplerFactory.h>
class GibbsSampler;

/**
 * @short Factory object for Gibbs Samplers
 */
class GibbsFactory : public SamplerFactory
{
  /**
   * Determines whether the factory can produce a GibbsSampler for
   * the given node.
   */
  virtual bool canSample(StochasticNode const *node) const = 0;
  /**
   * Returns a dynamically allocated Gibbs Sampler for a given node.
   */
  virtual GibbsSampler *makeGibbsSampler(StochasticNode *node) const = 0;
  /**
   * Implementation of SamplerFactory##makeSampler. This traverses
   * the graph, creating GibbsSampler, when possible, for each
   * individual node
   */
  void makeSampler(std::set<StochasticNode*> &nodes, 
		   std::list<Sampler*> &samplers) const;
};

#endif /* GIBBS_FACTORY_H */
