#include <config.h>
#include <sarray/LeftRangeIterator.h>
#include <stdexcept>

using std::out_of_range;

LeftRangeIterator::LeftRangeIterator(Range const &range)
  : Index(range.lower()), _range(range)
{
}

LeftRangeIterator &LeftRangeIterator::operator=(Index const &index)
{
  if (!_range.contains(index))
    throw out_of_range("Cannot set LeftRangeIterator: index out of range");
  
  Index::operator=(index);
  return *this;
}

bool LeftRangeIterator::atEnd() const
{
  int ndim = _range.ndim(false);
  return Index::operator[](ndim - 1) > _range.upper()[ndim - 1];
}

static void leftIncrement(Index &counter, Index const &lower, 
			  Index const &upper, int i)
{
  int ndim = counter.length();

  counter[i] += 1;
  if (counter[i] > upper[i]) {
    if (i < ndim - 1) {
      counter[i] = lower[i];
      leftIncrement(counter, lower, upper, i+1);
    }
  }
}

LeftRangeIterator &LeftRangeIterator::operator++()
{
  leftIncrement(*this, _range.lower(), _range.upper(), 0);
  return *this;
}
