#include <config.h>
#include <sarray/RightRangeIterator.h>
#include <stdexcept>

using std::out_of_range;

RightRangeIterator::RightRangeIterator(Range const &range)
  : Index(range.lower()), _range(range)
{
}

RightRangeIterator &RightRangeIterator::operator=(Index const &index)
{
  if (!_range.contains(index))
    throw out_of_range("Cannot set RightRangeIterator: index out of range");
  
  Index::operator=(index);
  return *this;
}

bool RightRangeIterator::atEnd() const
{
  return Index::operator[](0) > _range.upper()[0];
}

static void rightIncrement(Index &counter, Index const &lower, 
			   Index const &upper, int i)
{
  counter[i] += 1;
  if (counter[i] > upper[i]) {
    if (i > 0) {
      counter[i] = lower[i];
      rightIncrement(counter, lower, upper, i-1);
    }
  }
}

RightRangeIterator &RightRangeIterator::operator++()
{
  rightIncrement(*this, _range.lower(), _range.upper(), _range.ndim(false)-1);
  return *this;
}
