#include <config.h>
#include <sarray/SArray.h>

#include <stdexcept>
#include <cfloat>

const double JAGS_NA = -DBL_MAX*(1-1e-15);

using std::logic_error;
using std::length_error;

SArray::SArray(Index const &dim)
  : _range(Index(dim.length()), dim), _dim1(_range.dim(true)), _dim2(dim)
{
  _length = _range.length();
  _value = new double[_length];
  for (unsigned long i = 0; i < _length; ++i) {
    _value[i] = JAGS_NA;
  }
  _fixed = false;
}

SArray::SArray(SArray const &orig)
  : _range(orig._range), _dim1(orig._dim1), _dim2(orig._dim2)
{
  _length = orig.range().length();
  _value = new double[_length];
  for (unsigned long i = 0; i < _length; ++i) {
    _value[i] = orig._value[i];
  }
  _fixed = false;
}

SArray::~SArray()
{
  delete [] _value;
}

SArray & SArray::operator=(SArray const &rhs)
{
  _range = rhs._range;
  _dim1 = rhs._dim1;
  _dim2 = rhs._dim2;
  _fixed = rhs._fixed;
  _length = rhs._length;
  _value = new double[_length];
  for (unsigned long i = 0; i < _length; ++i) {
    _value[i] = rhs._value[i];
  }
  return *this;
}

/*
double const *SArray::value() const
{
  return _value;
}
*/

long SArray::length() const
{
  return _length;
}

bool SArray::isFixed() const
{
  return _fixed;
}

Index const &SArray::dim(bool drop) const
{
  if (drop) {
    return _dim1;
  }
  else {
    return _dim2;
  }
}

Range const &SArray::range() const
{
  return _range;
}

//debuggin
#include <iostream>

void SArray::setValue(double const *value, unsigned long length)
{
  if (_fixed) {
    throw logic_error ("Attempt to set value of fixed SArray");
  }
  else if (length != _length) {
    //debuggin
    std::cout << "Expected " << _length << ", got " << length << std::endl;
    throw length_error("Length mismatch error in SArray::setValue");
  }
  else {
    for (unsigned long i = 0; i < _length; ++i) {
      _value[i] = value[i];
    }
  }
}

void SArray::setFixed(bool fix)
{
  _fixed = fix;
}
