%{
#include <cstdio>
#include <iostream>
#include <fstream>
#include <map>
#define MATHLIB_STANDALONE
#include <Rmath.h>
#include <JAGSVersion.h>
#include <Console.h>
#include <model/ParseTree.h>
#include <sarray/SArray.h>
#include "ReadData.h"
  /* We need to include this when compiling with C++ 
     Start : */
#include <cstdlib>
#include <cstring>
  int zzerror(const char *);
  int zzlex();
  /* End : */
#define YYERROR_VERBOSE 0
  static Console *console;
  static bool interactive = true;
  std::map<std::string, SArray> _data_table;
  //  std::map<std::string, SArray> _init_table;
  bool open_rect_data_buffer(char *name);
  bool open_r_data_buffer(char *name);
  void return_to_main_buffer();
  void setMonitor(ParseTree *var, int thin);
  void clearMonitor(ParseTree *var);
  void doCoda (ParseTree *var, std::string const &stem);
  void doAllCoda (std::string const &stem);
  void doDump (std::string const &file, bool observed);

  static void setParameters(ParseTree *p, ParseTree *param1);
  static void setParameters(ParseTree *p, std::vector<ParseTree*> *parameters);
  static void setParameters(ParseTree *p, ParseTree *param1, ParseTree *param2);
%}

%defines
%name-prefix="zz"
%union
{
  int intval;
  double val;
  char string[100];
  ParseTree *ptree;
  std::vector<ParseTree*> *pvec;
}

%token <intval> INT
%token <val>    DOUBLE
%token <string> NAME
%token <string> STRING
%token <intval> ENDCMD

%token <intval> MODEL
%token <intval> DATA
%token <intval> IN 
%token <intval> TO
%token <intval> INITS
%token <intval> PARAMETERS

%token <intval> COMPILE
%token <intval> INITIALIZE
%token <intval> UPDATE

%token <intval> MONITOR
%token <intval> SET
%token <intval> CLEAR
%token <intval> THIN

%token <intval> CODA
%token <intval> STEM

%token <intval> SEED

%token <intval> EXIT
%token <intval> NCHAINS
%token <intval> CHAIN

%token <intval> LIST 
%token <intval> STRUCTURE
%token <intval> DOTDATA
%token <intval> DIM
%token <intval> NA
%token <intval> DIMNAMES
%token <intval> ITER
%token <intval> ARROW
%token <intval> '-'
%token <intval> ENDDATA
%token <intval> ASINTEGER

%type <ptree> var index 
%type <ptree> r_assignment r_structure
%type <ptree> range_element r_dim 
%type <ptree> r_value 
%type <pvec>  r_value_list r_assignment_list range_list
%type <ptree> r_value_collection r_integer_collection r_collection
%type <string> file_name;

%%

input: {if (interactive) std::cout << ". " << std::flush;}
| input line {if (interactive) std::cout << ". " << std::flush;}
;

line: ENDCMD {}
| command ENDCMD {}
| error ENDCMD {if(interactive) yyerrok; else exit(1);}
;

command: model 
| data_in
| data_to
| data_clear
| parameters_in
| parameters_to
| compile
| initialize
| update
| monitor
| coda
| seed
| exit
;

model: MODEL IN file_name {
  console->checkModel($3);
}
;

data_in: data r_assignment_list ENDDATA {
  readRData($2, _data_table);
  delete $2;
}
| data {}    // Failed to open the data file 
;

data_to: DATA TO file_name {
  doDump($3, true);
}
;

data: DATA IN file_name {
  if(!open_r_data_buffer($3)) {
    std::cerr << "Unable to open file " << $3 << std::endl << std::flush;
  }
}
;

data_clear: DATA CLEAR {
  std::cout << "Clearing data table " << std::endl;
  _data_table.clear();
}
;

parameters_in: parameters r_assignment_list ENDDATA
{
  std::map<std::string, SArray> init_table;
  readRData($2, init_table);
  delete $2;
  console->setParameters(init_table, 1);
}
| parameters r_assignment_list ENDDATA ',' CHAIN '(' INT ')' {
  std::map<std::string, SArray> init_table;
  readRData($2, init_table);
  delete $2;
  console->setParameters(init_table, $7);
}
| parameters {} // Failed to open the file
;

parameters_to: PARAMETERS TO file_name {
  doDump($3, false);
}
;

parameters: PARAMETERS IN file_name {
  if(!open_r_data_buffer($3)) {
    std::cerr << "Unable to open file " << $3 << std::endl << std::flush;
  }
}
| INITS IN file_name {
  /* Legacy option to not break existing scripts */
  if(!open_r_data_buffer($3)) {
    std::cerr << "Unable to open file " << $3 << std::endl << std::flush;
  }
}
;

compile: COMPILE {console->compile(_data_table, 1);}
| COMPILE ',' NCHAINS '(' INT ')' {
  console->compile(_data_table, $5);
}
;

initialize: INITIALIZE {
  console->initialize();
}
;

update: UPDATE INT {console->update($2);}
;

exit: EXIT {return 0;}
;

var: NAME {
  $$ = new ParseTree(P_VAR); $$->setName($1);
}
| NAME '[' range_list ']' {
  $$ = new ParseTree(P_VAR); $$->setName($1);
  setParameters($$, $3);
}
;

range_list: range_element {
  $$ = new std::vector<ParseTree*>(1, $1); 
}
| range_list ',' range_element {
  $$=$1; $$->push_back($3);
}
;

range_element: index {
  $$ = new ParseTree(P_RANGE); setParameters($$,$1);
}
| index ':' index {
  $$ = new ParseTree(P_RANGE); setParameters($$, $1, $3);
}
;

/* FIXME: Use integer value here */
index: INT {$$ = new ParseTree(P_VALUE); $$->setValue($1);}
;

monitor: monitor_set
| monitor_clear
;

monitor_set: MONITOR SET var  { setMonitor($3, 1);}
| MONITOR SET var ',' THIN '(' INT ')' { setMonitor($3, $7);}
;

monitor_clear: MONITOR CLEAR var {clearMonitor($3);}
;

/* 
   File names may optionally be enclosed in quotes, and this is required
   if the name includes spaces.
*/
file_name: NAME { sprintf($$,$1); }
| STRING {sprintf($$,$1);}
;

coda: CODA var {
  doCoda ($2, "jags");
}
| CODA var ',' STEM '(' file_name ')' {
  doCoda ($2, $6);
}
| CODA '*' {
  doAllCoda ("jags");
}
| CODA '*' ',' STEM '(' file_name ')' {
  doAllCoda ($6);
}
;

seed: SEED INT { console->setSeed(static_cast<unsigned int>($2)); }
;

/* Rules for scanning dumped R datasets */

r_assignment_list: r_assignment {
  $$ = new std::vector<ParseTree*>(1, $1);
}
| r_assignment_list r_assignment {
  $$ = $1; $$->push_back($2);
}
;

r_assignment: STRING ARROW r_structure {
  $$ = $3; $$->setName($1);
}
| STRING ARROW r_collection {
  $$ = new ParseTree(P_ARRAY);
  $$->setName($1);
  setParameters($$, $3);
}
;

r_structure: STRUCTURE '(' r_value_collection ',' r_dim ')' {
  $$ = new ParseTree(P_ARRAY); 
  setParameters($$, $3, $5);
}
| STRUCTURE '(' r_value_collection ',' r_dim ',' r_dimnames ')' {
  $$ = new ParseTree(P_ARRAY); 
  setParameters($$, $3, $5);
}
;

r_dim: DIM '=' r_collection {
  $$ = $3;
}
;

r_collection: r_integer_collection
| r_value_collection
;

r_integer_collection: ASINTEGER '(' r_value_collection ')' {
  $$ = $3;
}
;

r_value_collection: r_value { 
  $$ = new ParseTree(P_VECTOR); 
  setParameters($$, $1);
}
| 'c' '(' r_value_list ')' {
  $$ = new ParseTree(P_VECTOR);
  setParameters($$, $3);
}
;
 
r_value_list: r_value {$$ = new std::vector<ParseTree*>(1, $1); }
| r_value_list ',' r_value {$$ = $1; $$->push_back($3);}
;

r_value: DOUBLE {$$ = new ParseTree(P_VALUE); $$->setValue($1);}
| '-' DOUBLE {$$ = new ParseTree(P_VALUE); $$->setValue(-$2);}
| NA {$$ = new ParseTree(P_VALUE); $$->setValue(JAGS_NA);}
;

// Dimnames are currently parsed, but we do nothing with the result 

r_dimnames: DIMNAMES '=' r_name_list {;}
;

r_name_list: LIST 'c' '(' r_name_vector_list ')' {;}
;

r_name_vector_list: r_name_vector 
| r_name_vector_list ',' r_name_vector {;}
;

r_name_vector: 'c' '(' r_string_list ')' {;}
;

r_string_list: STRING {;}
| r_string_list ',' STRING {;} 
;
%%

int zzerror (const char *s)
{
  return_to_main_buffer();
  fprintf(stderr, "%s\n", s);
  return 0;
}

int main (int argc, char **argv)
{
  extern FILE *zzin;

  FILE *cmdfile = 0;
  if (argc > 2) {
    std::cerr << "Too many arguments" << std::endl;
  }
  else if (argc == 2) {
    interactive = false;
    cmdfile = fopen(argv[1],"r");
    if (cmdfile) {
      zzin = cmdfile;
    }
    else {
      std::cerr << "Unable to open command file " << argv[1] << std::endl;
      return 1;
    }
  }

  time_t t;
  time(&t);
  std::cout << "Welcome to JAGS " << JAGS_VERSION << " on " << ctime(&t);
  unsigned int seed = static_cast<unsigned int>(t);
  std::cout << "(seed = " << seed << ")" << std::endl;
  std::cout << "JAGS is free software and comes with ABSOLUTELY NO WARRANTY" 
            << std::endl;
  console = new Console(std::cout, std::cerr);
  console->setSeed(seed);
  zzparse();
  if (argc==2) {
    fclose(cmdfile);
  }
  delete console;
}

static Range getRange(ParseTree const *var)
{
  /* 
     Blank arguments, e.g. foo[] or bar[,1]  are not allowed.
  */
  unsigned int size = var->parameters().size();

  Index ind_lower(size), ind_upper(size);
  for (unsigned int i = 0; i < size; ++i) {
    ParseTree const *range_element = var->parameters()[i];
    switch(range_element->parameters().size()) {
    case 1:
      ind_lower[i] = (long) (range_element->parameters()[0]->value() + 1.0E-6);
      ind_upper[i] = ind_lower[i];
      break;
    case 2:
      ind_lower[i] = (long) (range_element->parameters()[0]->value() + 1.0E-6);  
      ind_lower[i] = (long) (range_element->parameters()[1]->value() + 1.0E-6);
      break;
    default:
      //Error! FIXME
      break;
    }
  }
  return Range(ind_lower, ind_upper);
}

void setMonitor(ParseTree *var, int thin)
{
  std::string const &name = var->name();
  if (var->parameters().empty()) {
    /* Requesting the whole node */
    console->setMonitor(name, thin);
  }
  else {
    /* Requesting subset of a multivariate node */
    Range range = getRange(var);
    console->setMonitor(name, range, thin);
  }
}

void clearMonitor(ParseTree *var)
{
  std::string const &name = var->name();
  if (var->parameters().empty()) {
    /* Requesting the whole node */
    console->clearMonitor(name);
  }
  else {
    /* Requesting subset of a multivariate node */
    Range range = getRange(var);
    console->clearMonitor(name, range);
  }
}

void doCoda (ParseTree *var, std::string const &stem)
{
  /* Open output file */
  std::string outname(stem); outname.append(".out"); 
  std::ofstream out(outname.c_str());
  if (!out) {
    std::cerr << "Failed to open file " << outname << std::endl;
    return;
  }
  /* Open indemax file */
  std::string indname(stem); indname.append(".ind"); 
  std::ofstream ind(indname.c_str());
  if (!ind) {
    std::cerr << "Failed to open file " << indname << std::endl;
    out.close();
    return;
  }

  std::string const &name = var->name();
  if (var->parameters().empty()) {
    /* Requesting the whole node */
    console->coda(name, out, ind);
  }
  else {
    /* Requesting subset of a multivariate node */
    Range range = getRange(var);
    console->coda(name, range, out, ind);
  }
  out.close();
  ind.close();
}

void doAllCoda (std::string const &stem)
{
  /* Open output file */
  std::string outname(stem); outname.append(".out"); 
  std::ofstream out(outname.c_str());
  if (!out) {
    std::cerr << "Failed to open file " << outname << std::endl;
    return;
  }
  /* Open indemax file */
  std::string indname(stem); indname.append(".ind"); 
  std::ofstream ind(indname.c_str());
  if (!ind) {
    std::cerr << "Failed to open file " << indname << std::endl;
    out.close();
    return;
  }

  console->coda(out, ind);

  out.close();
  ind.close();
}

void doDump(std::string const &file, bool observed)
{
  /* Open output file */
  std::ofstream out(file.c_str());
  if (!out) {
    std::cerr << "Failed to open file " << file << std::endl;
    return;
  }
  console->dumpState(out, 1, observed);
  out.close();
}

void setParameters(ParseTree *p, std::vector<ParseTree*> *parameters)
{
  /* 
     The parser dynamically allocates vectors of (pointers to)
     parameters. These vectors must be deleted when we are done with
     them.
  */
  p->setParameters(*parameters);
  delete parameters; 
}

void setParameters(ParseTree *p, ParseTree *param1)
{
  /*
    Wrapper function that creates a vector containing param1
    to be passed to ParseTree::setParameters.
  */
  std::vector<ParseTree *> parameters(1, param1);
  p->setParameters(parameters);
}

void setParameters(ParseTree *p, ParseTree *param1, ParseTree *param2)
{
  /*
    Wrapper function that creates a vector containing param1
    and param2, to be passed to ParseTree::setParameters
  */
  std::vector<ParseTree *> parameters;
  parameters.push_back(param1);
  parameters.push_back(param2);
  p->setParameters(parameters);
}
