%{
#include <iostream>
#include <cstring>
#include <model/ParseTree.h>
#include "parser.h"
  YY_BUFFER_STATE main_buffer = 0;
  void return_to_main_buffer();
%}

%option prefix="zz"

EXPONENT	[eE][+-][0-9]+
BRACKET         [ \t]*\(

%s RDATA
%x RECTDATA
%x COMMENT

%%

model                   zzlval.intval=MODEL; return MODEL;
data                    zzlval.intval=DATA; return DATA;
in                      zzlval.intval=IN; return IN;
to			zzlval.intval=TO; return TO;
inits                   zzlval.intval=INITS; return INITS;
parameters		zzlval.intval=PARAMETERS; return PARAMETERS;

compile			zzlval.intval=COMPILE; return COMPILE;
nchains                 zzlval.intval=NCHAINS; return NCHAINS;
initialize              zzlval.intval=INITIALIZE; return INITIALIZE;
chain                   zzlval.intval=CHAIN; return CHAIN;

update			zzlval.intval=UPDATE; return UPDATE;

monitor			zzlval.intval=MONITOR; return MONITOR;
set			zzlval.intval=SET; return SET;
clear			zzlval.intval=CLEAR; return CLEAR;
thin			zzlval.intval=THIN; return THIN;

coda			zzlval.intval=CODA; return CODA;
stem			zzlval.intval=STEM; return STEM;

seed                    zzlval.intval=SEED; return SEED;
exit			zzlval.intval=EXIT; return EXIT;


","			return ',';
"["			return '[';
"]"			return ']';
"("			return '(';
")"			return ')';
"="                     return '=';
"<-"			return ARROW;
"-"                     return '-';
"*"			return '*';
":"                     return ':';

"/*"                    BEGIN(COMMENT);
<COMMENT>[^*]*          /* Eat up anything that's not a '*'   */
<COMMENT>"*"+[^*/n]*    /* Eat up '*'s not followed by a '/'  */
<COMMENT>"*"+"/"        BEGIN(INITIAL);

<RDATA,RECTDATA,INITIAL>[ \t\r]+		/* Eat whitespace */
<RDATA,RECTDATA>[\n]+   /* Eat newlines */
<INITIAL>[\n]           return ENDCMD;



<INITIAL,RDATA,RECTDATA>([0-9]+){EXPONENT}  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<INITIAL,RDATA,RECTDATA>([0-9]+"."[0-9]*){EXPONENT}  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<INITIAL,RDATA,RECTDATA>([0-9]+"."[0-9]*)  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<INITIAL,RDATA,RECTDATA>("."[0-9]+){EXPONENT}  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<INITIAL,RDATA,RECTDATA>("."[0-9]+)  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<RDATA,RECTDATA>[0-9]+  {
  zzlval.val = atof(zztext); return DOUBLE;
}
<INITIAL>[0-9]+	{
  zzlval.intval = atoi(zztext); return INT;
}

<RDATA>".Dimnames"              return DIMNAMES;
<RDATA>".Data"                  return DOTDATA;
<RDATA>".Dim"                   return DIM;
<RDATA,RECTDATA>"NA"		return NA;
<RDATA>"structure"/{BRACKET}	return STRUCTURE;
<RDATA>"list"/{BRACKET}		return LIST;
<RDATA>"c"/{BRACKET} 		return 'c';
<RDATA>"as.integer"/{BRACKET}   return ASINTEGER;

[a-zA-Z]+[a-zA-Z0-9\._\-\\\/]* { 
  strcpy(zzlval.string, zztext);
  return NAME;
}

\"[^\"]*\"  {
  char buf[100];
  strncpy(buf, zztext, 100);
  strncpy(zzlval.string, &buf[1], strlen(buf) - 2);
  zzlval.string[strlen(buf) - 2] = '\0';
  //printf("%s\n", zzlval.string);
  return STRING;
}

<INITIAL><<EOF>>		yyterminate();
<RDATA,RECTDATA><<EOF>> {
  return_to_main_buffer();
  return ENDDATA;
}
%%

int zzwrap()
{
  return 1;
}

bool open_data_buffer(char *name, int start) {
  if (main_buffer) {
    return false;
  }
  FILE *file = fopen(name,"r");
  if (file) {
    main_buffer = YY_CURRENT_BUFFER;
    yy_switch_to_buffer(yy_create_buffer(file, YY_BUF_SIZE ) );
    BEGIN(start);
    return true;
  }
  else {
    return false;
  }
}

bool open_rect_data_buffer(char *name) {
  return open_data_buffer(name, RECTDATA);
}

bool open_r_data_buffer(char *name) {
  return open_data_buffer(name, RDATA);
}

void return_to_main_buffer() {
  if (main_buffer && (main_buffer != YY_CURRENT_BUFFER)) {
    yy_delete_buffer (YY_CURRENT_BUFFER);
    yy_switch_to_buffer (main_buffer);
    main_buffer = 0;
  }
  BEGIN(INITIAL);  
}

/*
([0-9]+"."[0-9]*){EXPONENT}	zzlval.val = atof(zztext); return DOUBLE;
([0-9]+"."[0-9]*)		zzlval.val = atof(zztext); return DOUBLE;
("."[0-9]+){EXPONENT}		zzlval.val = atof(zztext); return DOUBLE;
("."[0-9]+)			zzlval.val = atof(zztext); return DOUBLE;
[0-9]+				zzlval.intval = atoi(zztext); return INT;
*/
